"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImportCommonJS = void 0;
const tslib_1 = require("tslib");
const typescript_transformer_common_1 = require("@build-script/typescript-transformer-common");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
class ImportCommonJS extends typescript_transformer_common_1.NodeReplacer {
    resolver;
    kinds = [typescript_1.default.SyntaxKind.ImportDeclaration, typescript_1.default.SyntaxKind.ExportDeclaration];
    constructor(resolver) {
        super();
        this.resolver = resolver;
    }
    _check(node) {
        // logger.debug('[replacer/import]', ts.SyntaxKind[node.kind], node.getText());
        if (!(0, typescript_transformer_common_1.isImportExportFrom)(node)) {
            return false;
        }
        if (typescript_1.default.isExportDeclaration(node)) {
            this.logger.debug('            - isExportDeclaration');
            if (node.isTypeOnly || !node.exportClause || !typescript_1.default.isNamedExports(node.exportClause)) {
                this.logger.debug('                should skip');
                return false;
            }
        }
        else {
            this.logger.debug('            - !isExportDeclaration');
            if (!node.importClause ||
                node.importClause.isTypeOnly ||
                !node.importClause.namedBindings ||
                !typescript_1.default.isNamedImports(node.importClause.namedBindings)) {
                this.logger.debug('                should skip');
                return false;
            }
        }
        const self = node.getSourceFile().fileName;
        const id = node.moduleSpecifier.text;
        const out = this.resolver.resolve(self, id);
        if (!out.isNodeModules) {
            this.logger.debug('                skip node_modules');
            return false;
        }
        const pkgJson = out.readPackageJson();
        if (('' + pkgJson.type).toLowerCase() === 'module' || pkgJson.module) {
            this.logger.debug(`importing esnext module: ${id}`);
            return false;
        }
        this.logger.debug(`importing commonjs module: ${id}`);
        return true;
    }
    _replace(node, { factory }, logger) {
        const uniqueName = '_' + node.moduleSpecifier.text.replace(/^@/, '').replace(/[/\.-]/g, '_') + '_' + node.getStart();
        logger.debug(`modify import to {default} with id: ${uniqueName}`);
        if (typescript_1.default.isExportDeclaration(node)) {
            const uid = factory.createIdentifier(uniqueName);
            const elements = node.exportClause.elements;
            const list = elements.map((item) => {
                return factory.createVariableDeclaration(item.name, undefined, undefined, factory.createPropertyAccessExpression(uid, item.propertyName || item.name));
            });
            return [
                factory.createImportDeclaration(undefined, factory.createImportClause(false, uid, undefined), node.moduleSpecifier, undefined),
                factory.createVariableStatement([factory.createModifier(typescript_1.default.SyntaxKind.ExportKeyword)], factory.createVariableDeclarationList(list, typescript_1.default.NodeFlags.Const)),
            ];
        }
        else {
            const nameToUse = node.importClause?.name?.text ?? uniqueName;
            const uid = factory.createIdentifier(nameToUse);
            if (node.importClause)
                typescript_1.default.setOriginalNode(uid, node.importClause.name);
            return [
                factory.createImportDeclaration(node.modifiers, factory.createImportClause(false, uid, undefined), node.moduleSpecifier, node.assertClause),
                factory.createVariableStatement(undefined, factory.createVariableDeclarationList([
                    factory.createVariableDeclaration(factory.createObjectBindingPattern(node.importClause.namedBindings.elements.map((node) => {
                        return factory.createBindingElement(undefined, node.propertyName, node.name.text, undefined);
                    })), undefined, undefined, uid),
                ], typescript_1.default.NodeFlags.Const)),
            ];
        }
    }
}
exports.ImportCommonJS = ImportCommonJS;
//# sourceMappingURL=importCommonJS.js.map